@@ -48,15 +48,20 @@
 #include <fluent-bit/flb_upstream.h>
 #include <fluent-bit/flb_upstream_ha.h>
 
+#include <cmetrics/cmetrics.h>
+#include <cmetrics/cmt_counter.h>
+
 #ifdef FLB_HAVE_REGEX
 #include <fluent-bit/flb_regex.h>
 #endif
 
 /* Output plugin masks */
-#define FLB_OUTPUT_NET           32  /* output address may set host and port */
-#define FLB_OUTPUT_PLUGIN_CORE    0
-#define FLB_OUTPUT_PLUGIN_PROXY   1
-#define FLB_OUTPUT_NO_MULTIPLEX 512
+#define FLB_OUTPUT_NET            32  /* output address may set host and port */
+#define FLB_OUTPUT_PLUGIN_CORE     0
+#define FLB_OUTPUT_PLUGIN_PROXY    1
+#define FLB_OUTPUT_NO_MULTIPLEX  512
+#define FLB_OUTPUT_PRIVATE      1024
+
 
 /* Event type handlers */
 #define FLB_OUTPUT_LOGS        1
@@ -307,6 +312,20 @@ struct flb_output_instance {
 
     struct mk_list _head;                /* link to config->inputs       */
 
+    /*
+     * CMetrics
+     * --------
+     */
+    struct cmt *cmt;                         /* parent context            */
+    struct cmt_counter *cmt_proc_records;    /* m: output_proc_records    */
+    struct cmt_counter *cmt_proc_bytes;      /* m: output_proc_bytes      */
+    struct cmt_counter *cmt_errors;          /* m: output_errors          */
+    struct cmt_counter *cmt_retries;         /* m: output_retries         */
+    struct cmt_counter *cmt_retries_failed;  /* m: output_retries_failed  */
+    struct cmt_counter *cmt_dropped_records; /* m: output_dropped_records */
+    struct cmt_counter *cmt_retried_records; /* m: output_retried_records */
+
+    /* OLD Metrics API */
 #ifdef FLB_HAVE_METRICS
     struct flb_metrics *metrics;         /* metrics                      */
 #endif
@@ -695,7 +714,8 @@ struct flb_output_instance *flb_output_get_instance(struct flb_config *config,
 int flb_output_flush_finished(struct flb_config *config, int out_id);
 
 struct flb_output_instance *flb_output_new(struct flb_config *config,
-                                           const char *output, void *data);
+                                           const char *output, void *data,
+                                           int public_only);
 const char *flb_output_name(struct flb_output_instance *in);
 int flb_output_set_property(struct flb_output_instance *out,
                             const char *k, const char *v);
