@@ -21,15 +21,17 @@
 #include <stdarg.h>
 #include <string.h>
 #include <errno.h>
-#include <pthread.h>
 #include <signal.h>
 
+#include <mk_core/mk_pthread.h>
+
 #include <monkey/mk_lib.h>
 #include <monkey/monkey.h>
 #include <monkey/mk_stream.h>
 #include <monkey/mk_thread.h>
 #include <monkey/mk_scheduler.h>
 #include <monkey/mk_fifo.h>
+#include <monkey/mk_utils.h>
 
 #define config_eq(a, b) strcasecmp(a, b)
 
@@ -189,7 +191,17 @@ int mk_start(mk_ctx_t *ctx)
     mk_event_wait(server->lib_evl);
     mk_event_foreach(event, server->lib_evl) {
         fd = event->fd;
+
+        /* When using libevent _mk_event_channel_create creates a unix socket
+         * instead of a pipe and windows doesn't us calling read / write on a
+         * socket instead of recv / send
+         */
+#ifdef _WIN32        
+        bytes = recv(fd, &val, sizeof(uint64_t), MSG_WAITALL);
+#else
         bytes = read(fd, &val, sizeof(uint64_t));
+#endif
+        
         if (bytes <= 0) {
             return -1;
         }
@@ -251,7 +263,7 @@ int mk_config_set_property(struct mk_server *server, char *k, char *v)
     else if (config_eq(k, "Workers") == 0) {
         num = atoi(v);
         if (num <= 0) {
-            server->workers = sysconf(_SC_NPROCESSORS_ONLN);
+            server->workers = mk_utils_get_system_core_count();
         }
         else {
             server->workers = num;
@@ -364,7 +376,7 @@ int mk_config_set(mk_ctx_t *ctx, ...)
 }
 
 /* Given a vhost id, return the vhost context */
-static struct mk_vhost *mk_vhost_lookup(mk_ctx_t *ctx, int id)
+struct mk_vhost *mk_vhost_lookup(mk_ctx_t *ctx, int id)
 {
     struct mk_vhost *host;
     struct mk_list *head;
