@@ -452,7 +452,6 @@ static struct addrinfo *flb_net_translate_ares_addrinfo(struct ares_addrinfo *in
     struct ares_addrinfo_node *current_ares_record;
     struct addrinfo *previous_output_record;
     struct addrinfo *current_output_record;
-    struct addrinfo *next_output_record;
     struct addrinfo *output;
     int failure_detected;
 
@@ -527,15 +526,17 @@ static void flb_net_getaddrinfo_callback(void *arg, int status, int timeouts,
         context->result = flb_net_translate_ares_addrinfo(res);
 
         if (NULL == context->result) {
-            context->result_code = 2;
+            /* Currently, translation fails when malloc error occured. */
+            context->result_code = EAI_MEMORY;
         }
         else {
-            context->result_code = 0;
+            context->result_code = ARES_SUCCESS;
         }
+
         ares_freeaddrinfo(res);
     }
     else {
-        context->result_code = 1;
+        context->result_code = status;
     }
 
     context->finished = 1;
@@ -780,7 +781,13 @@ flb_sockfd_t flb_net_tcp_connect(const char *host, unsigned long port,
     }
 
     if (ret != 0) {
-        flb_warn("[net] getaddrinfo(host='%s'): %s", host, gai_strerror(ret));
+        if (is_async) {
+            flb_warn("[net] getaddrinfo(host='%s', err=%d): %s", host, ret, ares_strerror(ret));
+        }
+        else {
+            flb_warn("[net] getaddrinfo(host='%s', err=%d): %s", host, ret, gai_strerror(ret));
+        }
+
         return -1;
     }
 
