@@ -178,13 +178,15 @@ int flb_unescape_string_utf8(const char *in_buf, int sz, char *out_buf)
                     esc_in = u8_read_escape_sequence(next, size, &ch) + 1;
                 }
                 else {
-                    ch = (uint32_t) *in_buf;
+                    /* because char is unsigned char by default on arm, so we need to do a explicit conversion */
+                    ch = (uint32_t) (signed char) *in_buf;
                     esc_in = 1;
                 }
             }
         }
         else {
-            ch = (uint32_t) *in_buf;
+            /* explicit convert char to signed char */
+            ch = (uint32_t) (signed char) *in_buf;
             esc_in = 1;
         }
 
