@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2020 The Fluent Bit Authors
+ *  Copyright (C) 2019-2021 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -35,18 +35,11 @@ FLB_TLS_DEFINE(struct flb_worker, flb_worker_ctx);
  */
 static void step_callback(void *data)
 {
-    int ret;
     struct flb_worker *worker = data;
 
     /* Set the worker context global */
     FLB_TLS_SET(flb_worker_ctx, worker);
 
-    /* Initialize worker-required data */
-    ret = flb_log_worker_init(worker);
-    if (ret == -1) {
-        fprintf(stderr, "[worker] error initializing log-worker context\n");
-    }
-
     /* not too scary :) */
     worker->func(worker->data);
 
@@ -54,6 +47,25 @@ static void step_callback(void *data)
     pthread_exit(NULL);
 }
 
+struct flb_worker *flb_worker_context_create(void (*func) (void *), void *arg,
+                                             struct flb_config *config)
+{
+    struct flb_worker *worker;
+
+    worker = flb_calloc(1, sizeof(struct flb_worker));
+    if (!worker) {
+        flb_errno();
+        return NULL;
+    }
+    MK_EVENT_ZERO(&worker->event);
+    worker->func   = func;
+    worker->data   = arg;
+    worker->config = config;
+    worker->log_ctx = config->log;
+
+    return worker;
+}
+
 /*
  * Creates a worker (POSIX thread). This function creates a worker
  * context and also setup the 'step' callback to initialize generic
@@ -69,17 +81,10 @@ int flb_worker_create(void (*func) (void *), void *arg, pthread_t *tid,
     int ret;
     struct flb_worker *worker;
 
-    worker = flb_malloc(sizeof(struct flb_worker));
+    worker = flb_worker_context_create(func, arg, config);
     if (!worker) {
-        perror("malloc");
         return -1;
     }
-    MK_EVENT_ZERO(&worker->event);
-
-    worker->func   = func;
-    worker->data   = arg;
-    worker->config = config;
-    worker->log_ctx = config->log;
 
     /* Initialize log-specific */
     ret = flb_log_worker_init(worker);
@@ -132,6 +137,16 @@ struct flb_worker *flb_worker_get()
     return FLB_TLS_GET(flb_worker_ctx);
 }
 
+void flb_worker_destroy(struct flb_worker *worker)
+{
+    if (!worker) {
+        return;
+    }
+
+    mk_list_del(&worker->_head);
+    flb_free(worker);
+}
+
 int flb_worker_exit(struct flb_config *config)
 {
     int c = 0;
@@ -141,8 +156,7 @@ int flb_worker_exit(struct flb_config *config)
 
     mk_list_foreach_safe(head, tmp, &config->workers) {
         worker = mk_list_entry(head, struct flb_worker, _head);
-        mk_list_del(&worker->_head);
-        flb_free(worker);
+        flb_worker_destroy(worker);
         c++;
     }
 
