@@ -22,15 +22,15 @@
 #include <fluent-bit/stream_processor/flb_sp_window.h>
 #include <fluent-bit/stream_processor/flb_sp_parser.h>
 #include <fluent-bit/stream_processor/flb_sp_groupby.h>
+#include <fluent-bit/stream_processor/flb_sp_aggregate_func.h>
 
 void flb_sp_window_prune(struct flb_sp_task *task)
 {
     int i;
-    int key_id;
     int map_entries;
     rb_result_t result;
-    struct aggr_node *aggr_node;
-    struct aggr_node *aggr_node_hs;
+    struct aggregate_node *aggr_node;
+    struct aggregate_node *aggr_node_hs;
     struct mk_list *tmp;
     struct mk_list *head;
     struct flb_sp_hopping_slot *hs;
@@ -42,15 +42,15 @@ void flb_sp_window_prune(struct flb_sp_task *task)
     case FLB_SP_WINDOW_DEFAULT:
     case FLB_SP_WINDOW_TUMBLING:
         if (task->window.records > 0) {
-            mk_list_foreach_safe(head, tmp, &task->window.aggr_list) {
-                aggr_node = mk_list_entry(head, struct aggr_node, _head);
+            mk_list_foreach_safe(head, tmp, &task->window.aggregate_list) {
+                aggr_node = mk_list_entry(head, struct aggregate_node, _head);
                 mk_list_del(&aggr_node->_head);
-                flb_sp_aggr_node_destroy(cmd, aggr_node);
+                flb_sp_aggregate_node_destroy(cmd, aggr_node);
             }
 
-            rb_tree_destroy(&task->window.aggr_tree);
-            mk_list_init(&task->window.aggr_list);
-            rb_tree_new(&task->window.aggr_tree, flb_sp_groupby_compare);
+            rb_tree_destroy(&task->window.aggregate_tree);
+            mk_list_init(&task->window.aggregate_list);
+            rb_tree_new(&task->window.aggregate_tree, flb_sp_groupby_compare);
             task->window.records = 0;
         }
         break;
@@ -61,42 +61,26 @@ void flb_sp_window_prune(struct flb_sp_task *task)
 
         hs = mk_list_entry_first(&task->window.hopping_slot,
                                  struct flb_sp_hopping_slot, _head);
-        mk_list_foreach_safe(head, tmp, &task->window.aggr_list) {
-            aggr_node = mk_list_entry(head, struct aggr_node, _head);
-            result = rb_tree_find(&hs->aggr_tree, aggr_node, &rb_result);
+        mk_list_foreach_safe(head, tmp, &task->window.aggregate_list) {
+            aggr_node = mk_list_entry(head, struct aggregate_node, _head);
+            result = rb_tree_find(&hs->aggregate_tree, aggr_node, &rb_result);
             if (result == RB_OK) {
-                aggr_node_hs = mk_list_entry(rb_result, struct aggr_node, _rb_head);
+                aggr_node_hs = mk_list_entry(rb_result, struct aggregate_node, _rb_head);
                 if (aggr_node_hs->records == aggr_node->records) {
-                    rb_tree_remove(&task->window.aggr_tree, &aggr_node->_rb_head);
+                    rb_tree_remove(&task->window.aggregate_tree, &aggr_node->_rb_head);
                     mk_list_del(&aggr_node->_head);
                     // Destroy aggregation node
-                    flb_sp_aggr_node_destroy(cmd, aggr_node);
+                    flb_sp_aggregate_node_destroy(cmd, aggr_node);
                 }
                 else {
                     aggr_node->records -= aggr_node_hs->records;
                     map_entries = mk_list_size(&cmd->keys);
 
-                    key_id = 0;
                     ckey = mk_list_entry_first(&cmd->keys,
                                                struct flb_sp_cmd_key, _head);
                     for (i = 0; i < map_entries; i++) {
-                        switch (ckey->aggr_func) {
-                        case FLB_SP_AVG:
-                        case FLB_SP_SUM:
-                            if (aggr_node->nums[i].type == FLB_SP_NUM_I64) {
-                                aggr_node->nums[i].i64 -= aggr_node_hs->nums[i].i64;
-                            }
-                            else if (aggr_node->nums[i].type == FLB_SP_NUM_F64) {
-                                aggr_node->nums[i].f64 -= aggr_node_hs->nums[i].f64;
-                            }
-                            break;
-                        }
-
-                        if (!ckey->name && ckey->timeseries_func) {
-                            ckey->timeseries->cb_func_rem(aggr_node->ts[key_id],
-                                                          aggr_node_hs->ts[key_id],
-                                                          NULL);
-                            key_id++;
+                        if (ckey->aggr_func) {
+                            aggregate_func_remove[ckey->aggr_func - 1](aggr_node, aggr_node_hs, i);
                         }
 
                         ckey = mk_list_entry_next(&ckey->_head, struct flb_sp_cmd_key,
@@ -108,12 +92,12 @@ void flb_sp_window_prune(struct flb_sp_task *task)
         task->window.records -= hs->records;
 
         /* Destroy hopping slot */
-        mk_list_foreach_safe(head, tmp, &hs->aggr_list) {
-            aggr_node_hs = mk_list_entry(head, struct aggr_node, _head);
+        mk_list_foreach_safe(head, tmp, &hs->aggregate_list) {
+            aggr_node_hs = mk_list_entry(head, struct aggregate_node, _head);
             mk_list_del(&aggr_node_hs->_head);
-            flb_sp_aggr_node_destroy(cmd, aggr_node_hs);
+            flb_sp_aggregate_node_destroy(cmd, aggr_node_hs);
         }
-        rb_tree_destroy(&hs->aggr_tree);
+        rb_tree_destroy(&hs->aggregate_tree);
         mk_list_del(&hs->_head);
         flb_free(hs);
 
